@@ -1,5 +1,5 @@
 /*
- * $Id: Attribute.java 788065 2009-06-24 16:05:28Z apetrelli $
+ * $Id: Attribute.java 880948 2009-11-16 20:29:06Z apetrelli $
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -32,7 +32,7 @@ import java.util.Set;
 /**
  * Common implementation of attribute definition.
  *
- * @version $Rev: 788065 $ $Date: 2009-06-24 18:05:28 +0200 (mer, 24 giu 2009) $
+ * @version $Rev: 880948 $ $Date: 2009-11-16 21:29:06 +0100 (lun, 16 nov 2009) $
  */
 public class Attribute implements Serializable, Cloneable {
 
@@ -297,6 +297,32 @@ public class Attribute implements Serializable, Cloneable {
         return attribute;
     }
 
+    /**
+     * Creates a template attribute, starting from the name of the template.
+     *
+     * @param template The template that will be rendered.
+     * @param templateExpression The template expression that will be evaluated
+     * to a template.
+     * @param templateType The type, or renderer, of the template. If null, the
+     * default <code>template</code> will be used.
+     * @param role The comma-separated roles for which the template is
+     * authorized to be rendered.
+     * @return The template attribute.
+     * @since 2.2.2
+     */
+    public static Attribute createTemplateAttribute(String template,
+            String templateExpression, String templateType, String role) {
+        Attribute templateAttribute = createTemplateAttribute(template);
+        templateAttribute.setRole(role);
+        if (templateType != null) {
+            templateAttribute.setRenderer(templateType);
+        }
+        templateAttribute
+                .setExpressionObject(Expression
+                        .createExpressionFromDescribedExpression(templateExpression));
+        return templateAttribute;
+    }
+
     /**
      * Creates a template attribute, starting from the expression to evaluate to
      * obtain the template.
