@@ -1,5 +1,5 @@
 /*
- * $Id: ReloadableDefinitionsFactory.java 527870 2007-04-12 10:19:41Z apetrelli $
+ * $Id: ReloadableDefinitionsFactory.java 669657 2008-06-19 19:05:51Z apetrelli $
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -19,29 +19,13 @@
  * under the License.
  */
 
-
 package org.apache.tiles.definition;
 
 /**
- * Indicates support for reloading Tiles configuration when it changes.
+ * Indicates support for reloading Tiles configuration when it changes in {@link DefinitionsFactory}.
  *
- * @version $Rev: 527870 $ $Date: 2007-04-12 12:19:41 +0200 (Thu, 12 Apr 2007) $
+ * @version $Rev: 669657 $ $Date: 2008-06-19 21:05:51 +0200 (gio, 19 giu 2008) $
+ * @deprecated Use {@link Refreshable}.
  */
-public interface ReloadableDefinitionsFactory {
-
-    /**
-     * Indicates whether the DefinitionsFactory is out of date and needs to be
-     * reloaded.
-     *
-     * @return <code>true</code> if the definitions need to be refreshed.
-     */
-    boolean refreshRequired();
-
-    /**
-     * Refreshes the stored definitions, reloading them.
-     *
-     * @throws DefinitionsFactoryException If something goes wrong during
-     * refresh.
-     */
-    void refresh() throws DefinitionsFactoryException;
+public interface ReloadableDefinitionsFactory extends Refreshable {
 }
