@@ -1,5 +1,5 @@
 /*
- * $Id: UrlDefinitionsFactory.java 822631 2009-10-07 09:21:12Z apetrelli $
+ * $Id: UrlDefinitionsFactory.java 934508 2010-04-15 18:30:51Z apetrelli $
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -33,6 +33,7 @@ import org.apache.tiles.definition.dao.DefinitionDAO;
 import org.apache.tiles.definition.dao.ResolvingLocaleUrlDefinitionDAO;
 import org.apache.tiles.definition.dao.URLReader;
 import org.apache.tiles.impl.BasicTilesContainer;
+import org.apache.tiles.locale.impl.DefaultLocaleResolver;
 import org.apache.tiles.util.LocaleUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -48,7 +49,7 @@ import org.slf4j.LoggerFactory;
  * DigesterDefinitionsReader} class unless another implementation is specified.
  * </p>
  *
- * @version $Rev: 822631 $ $Date: 2009-10-07 11:21:12 +0200 (mer, 07 ott 2009) $
+ * @version $Rev: 934508 $ $Date: 2010-04-15 20:30:51 +0200 (gio, 15 apr 2010) $
  * @deprecated Use {@link UnresolvingLocaleDefinitionsFactory} and using
  * {@link ResolvingLocaleUrlDefinitionDAO} as Tiles DAO.
  */
@@ -164,10 +165,21 @@ public class UrlDefinitionsFactory extends UnresolvingLocaleDefinitionsFactory
      * @return The default definition DAO.
      * @since 2.1.0
      */
-    protected DefinitionDAO<Locale> createDefaultDefinitionDAO() {
+    protected ResolvingLocaleUrlDefinitionDAO createDefaultDefinitionDAO() {
         return new ResolvingLocaleUrlDefinitionDAO();
     }
 
+    @Override
+    public void init(Map<String, String> params) {
+        super.init(params);
+        setLocaleResolver(new DefaultLocaleResolver());
+        ResolvingLocaleUrlDefinitionDAO dao = createDefaultDefinitionDAO();
+        dao.setApplicationContext(applicationContext);
+        dao.init(params);
+        setDefinitionDAO(dao);
+
+    }
+
     /**
      * Creates and returns a {@link Definitions} set by reading
      * configuration data from the applied sources.
