@@ -1,5 +1,5 @@
 /*
- * $Id: BasicTilesContainer.java 619574 2008-02-07 19:09:33Z apetrelli $
+ * $Id: BasicTilesContainer.java 797765 2009-07-25 13:20:26Z apetrelli $
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -20,35 +20,35 @@
  */
 package org.apache.tiles.impl;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import java.io.IOException;
+import java.io.Writer;
+import java.util.ArrayList;
+import java.util.List;
+import java.util.Map;
+import java.util.StringTokenizer;
+
+import org.apache.tiles.ArrayStack;
 import org.apache.tiles.Attribute;
 import org.apache.tiles.AttributeContext;
+import org.apache.tiles.BasicAttributeContext;
 import org.apache.tiles.Definition;
 import org.apache.tiles.TilesApplicationContext;
 import org.apache.tiles.TilesContainer;
-import org.apache.tiles.TilesException;
-import org.apache.tiles.Attribute.AttributeType;
-import org.apache.tiles.context.BasicAttributeContext;
-import org.apache.tiles.context.TilesContextFactory;
 import org.apache.tiles.context.TilesRequestContext;
+import org.apache.tiles.context.TilesRequestContextFactory;
 import org.apache.tiles.definition.DefinitionsFactory;
 import org.apache.tiles.definition.DefinitionsFactoryException;
 import org.apache.tiles.definition.NoSuchDefinitionException;
+import org.apache.tiles.evaluator.AttributeEvaluator;
+import org.apache.tiles.evaluator.AttributeEvaluatorFactory;
+import org.apache.tiles.evaluator.AttributeEvaluatorFactoryAware;
 import org.apache.tiles.preparer.NoSuchPreparerException;
 import org.apache.tiles.preparer.PreparerFactory;
 import org.apache.tiles.preparer.ViewPreparer;
-
-import java.io.IOException;
-import java.io.Writer;
-import java.net.URL;
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-import java.util.Stack;
-import java.util.StringTokenizer;
+import org.apache.tiles.renderer.AttributeRenderer;
+import org.apache.tiles.renderer.RendererFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Basic implementation of the tiles container interface.
@@ -57,13 +57,17 @@ import java.util.StringTokenizer;
  * override the container
  *
  * @since 2.0
- * @version $Rev: 619574 $ $Date: 2008-02-07 20:09:33 +0100 (Thu, 07 Feb 2008) $
+ * @version $Rev: 797765 $ $Date: 2009-07-25 15:20:26 +0200 (sab, 25 lug 2009) $
  */
-public class BasicTilesContainer implements TilesContainer {
+public class BasicTilesContainer implements TilesContainer,
+        AttributeEvaluatorFactoryAware {
 
     /**
-     * Constant representing the configuration parameter
-     * used to define the tiles definition resources.
+     * Constant representing the configuration parameter used to define the
+     * tiles definition resources.
+     *
+     * @deprecated Use
+     * {@link org.apache.tiles.definition.DefinitionsFactory#DEFINITIONS_CONFIG}.
      */
     public static final String DEFINITIONS_CONFIG = "org.apache.tiles.impl.BasicTilesContainer.DEFINITIONS_CONFIG";
 
@@ -84,8 +88,8 @@ public class BasicTilesContainer implements TilesContainer {
      * Log instance for all BasicTilesContainer
      * instances.
      */
-    private static final Log LOG =
-        LogFactory.getLog(BasicTilesContainer.class);
+    private final Logger log = LoggerFactory
+            .getLogger(BasicTilesContainer.class);
 
     /**
      * The Tiles application context object.
@@ -103,9 +107,19 @@ public class BasicTilesContainer implements TilesContainer {
     private PreparerFactory preparerFactory;
 
     /**
-     * The Tiles context factory.
+     * The renderer factory.
      */
-    private TilesContextFactory contextFactory;
+    private RendererFactory rendererFactory;
+
+    /**
+     * The attribute evaluator.
+     */
+    private AttributeEvaluatorFactory attributeEvaluatorFactory;
+
+    /**
+     * The Tiles request context factory.
+     */
+    private TilesRequestContextFactory contextFactory;
 
     /**
      * Initialization flag. If set, this container cannot be changed.
@@ -116,19 +130,33 @@ public class BasicTilesContainer implements TilesContainer {
      * Initialize the Container with the given configuration.
      *
      * @param initParameters application context for this container
-     * @throws TilesException If something goes wrong during initialization.
+     * @throws IllegalStateException If the container has been already
+     * initialized.
+     * @throws DefinitionsFactoryException If something goes wrong during
+     * initialization.
      */
-    public void init(Map<String, String> initParameters) throws TilesException {
+    public void init(Map<String, String> initParameters) {
         checkInit();
         initialized = true;
-        if (LOG.isInfoEnabled()) {
-            LOG.info("Initializing Tiles2 container. . .");
-        }
 
-        //Everything is now initialized.  We will populate
-        // our definitions
-        initializeDefinitionsFactory(definitionsFactory, getResourceString(),
-                initParameters);
+        if (rendererFactory == null) {
+            throw new IllegalStateException("RendererFactory not specified");
+        }
+        if (preparerFactory == null) {
+            throw new IllegalStateException("PreparerFactory not specified");
+        }
+        if (definitionsFactory == null) {
+            throw new IllegalStateException("DefinitionsFactory not specified");
+        }
+        if (attributeEvaluatorFactory == null) {
+            throw new IllegalStateException("AttributeEvaluatorFactory not specified");
+        }
+        if (contextFactory == null) {
+            throw new IllegalStateException("TilesContextFactory not specified");
+        }
+        if (context == null) {
+            throw new IllegalStateException("TilesApplicationContext not specified");
+        }
     }
 
     /** {@inheritDoc} */
@@ -143,6 +171,14 @@ public class BasicTilesContainer implements TilesContainer {
         endContext(tilesContext);
     }
 
+    /** {@inheritDoc} */
+    public void renderContext(Object... requestItems) {
+        TilesRequestContext request = getRequestContext(requestItems);
+        AttributeContext attributeContext = getAttributeContext(request);
+
+        render(request, attributeContext);
+    }
+
     /**
      * Returns the Tiles application context used by this container.
      *
@@ -171,9 +207,22 @@ public class BasicTilesContainer implements TilesContainer {
     /**
      * Returns the context factory.
      *
-     * @return The context factory.
+     * @return Always <code>null</code>.
+     * @deprecated Do not use it, it returns <code>null</code>. Use
+     * {@link #getRequestContextFactory()}.
      */
-    public TilesContextFactory getContextFactory() {
+    @Deprecated
+    public org.apache.tiles.context.TilesContextFactory getContextFactory() {
+        return null;
+    }
+
+    /**
+     * Returns the request context factory.
+     *
+     * @return The request context factory.
+     * @since 2.1.1
+     */
+    protected TilesRequestContextFactory getRequestContextFactory() {
         return contextFactory;
     }
 
@@ -181,8 +230,20 @@ public class BasicTilesContainer implements TilesContainer {
      * Sets the context factory.
      *
      * @param contextFactory The context factory.
+     * @deprecated Use
+     * {@link #setRequestContextFactory(TilesRequestContextFactory)}.
      */
-    public void setContextFactory(TilesContextFactory contextFactory) {
+    public void setContextFactory(org.apache.tiles.context.TilesContextFactory contextFactory) {
+        // Does nothing
+    }
+
+    /**
+     * Sets the request context factory.
+     *
+     * @param contextFactory The context factory.
+     * @since 2.1.1
+     */
+    public void setRequestContextFactory(TilesRequestContextFactory contextFactory) {
         checkInit();
         this.contextFactory = contextFactory;
     }
@@ -226,10 +287,25 @@ public class BasicTilesContainer implements TilesContainer {
         this.preparerFactory = preparerFactory;
     }
 
+    /**
+     * Sets the renderer instance factory.
+     *
+     * @param rendererFactory the renderer instance factory for this container.
+     * @since 2.1.0
+     */
+    public void setRendererFactory(RendererFactory rendererFactory) {
+        this.rendererFactory = rendererFactory;
+    }
+
     /** {@inheritDoc} */
-    public void prepare(String preparer, Object... requestItems)
-        throws TilesException {
-        TilesRequestContext requestContext = getContextFactory().createRequestContext(
+    public void setAttributeEvaluatorFactory(
+            AttributeEvaluatorFactory attributeEvaluatorFactory) {
+        this.attributeEvaluatorFactory = attributeEvaluatorFactory;
+    }
+
+    /** {@inheritDoc} */
+    public void prepare(String preparer, Object... requestItems) {
+        TilesRequestContext requestContext = getRequestContextFactory().createRequestContext(
             getApplicationContext(),
             requestItems
         );
@@ -237,9 +313,8 @@ public class BasicTilesContainer implements TilesContainer {
     }
 
     /** {@inheritDoc} */
-    public void render(String definitionName, Object... requestItems)
-        throws TilesException {
-        TilesRequestContext requestContext = getContextFactory().createRequestContext(
+    public void render(String definitionName, Object... requestItems) {
+        TilesRequestContext requestContext = getRequestContextFactory().createRequestContext(
             getApplicationContext(),
             requestItems
         );
@@ -247,46 +322,27 @@ public class BasicTilesContainer implements TilesContainer {
     }
 
     /** {@inheritDoc} */
+    @Deprecated
     public void render(Attribute attr, Writer writer, Object... requestItems)
-        throws TilesException, IOException {
-        TilesRequestContext request = getRequestContext(requestItems);
+        throws IOException {
+        render(attr, requestItems);
+    }
 
-        if (attr == null) {
-            throw new TilesException("Cannot render a null attribute");
-        }
+    /** {@inheritDoc} */
+    public void render(Attribute attr, Object... requestItems)
+        throws IOException {
+        TilesRequestContext requestContext = getRequestContextFactory()
+                .createRequestContext(getApplicationContext(), requestItems);
+        render(attr, requestContext);
+    }
 
-        if (!isPermitted(request, attr.getRoles())) {
-            if (LOG.isDebugEnabled()) {
-                LOG.debug("Access to attribute denied.  User not in role '"
-                        + attr.getRoles() + "'");
-            }
-            return;
-        }
-
-        AttributeType type = attr.getType();
-        if (type == null) {
-            type = calculateType(attr, request);
-            attr.setType(type);
-        }
-
-        switch (type) {
-            case OBJECT:
-                throw new TilesException(
-                    "Cannot insert an attribute of 'object' type");
-            case STRING:
-                writer.write(attr.getValue().toString());
-                break;
-            case DEFINITION:
-                render(request, attr.getValue().toString());
-                break;
-            case TEMPLATE:
-                request.dispatch(attr.getValue().toString());
-                break;
-            default: // should not happen
-                throw new TilesException(
-                        "Unrecognized type for attribute value "
-                        + attr.getValue());
-        }
+    /** {@inheritDoc} */
+    public Object evaluate(Attribute attribute, Object... requestItems) {
+        TilesRequestContext request = getRequestContextFactory()
+                .createRequestContext(context, requestItems);
+        AttributeEvaluator evaluator = attributeEvaluatorFactory
+                .getAttributeEvaluator(attribute);
+        return evaluator.evaluate(attribute, request);
     }
 
     /** {@inheritDoc} */
@@ -304,7 +360,7 @@ public class BasicTilesContainer implements TilesContainer {
      * exception.
      */
     protected Definition getDefinition(String definitionName,
-            TilesRequestContext request) throws DefinitionsFactoryException {
+            TilesRequestContext request) {
         Definition definition =
             definitionsFactory.getDefinition(definitionName, request);
         return definition;
@@ -379,34 +435,22 @@ public class BasicTilesContainer implements TilesContainer {
      * @param resourceString The string containing a comma-separated-list of
      * resources.
      * @param initParameters A map containing the initialization parameters.
-     * @throws TilesException If something goes wrong.
+     * @throws DefinitionsFactoryException If something goes wrong.
+     * @deprecated Do not use, the Definitions Factory should be initialized by
+     * the Tiles Container Factory.
      */
+    @Deprecated
     protected void initializeDefinitionsFactory(
             DefinitionsFactory definitionsFactory, String resourceString,
-            Map<String, String> initParameters) throws TilesException {
-        List<String> resources = getResourceNames(resourceString);
-
-        try {
-            for (String resource : resources) {
-                URL resourceUrl = context.getResource(resource);
-                if (resourceUrl != null) {
-                    if (LOG.isDebugEnabled()) {
-                        LOG.debug("Adding resource '" + resourceUrl + "' to definitions factory.");
-                    }
-                    definitionsFactory.addSource(resourceUrl);
-                } else {
-                    LOG.warn("Unable to find configured definition '" + resource + "'");
-                }
-            }
-        } catch (IOException e) {
-            throw new DefinitionsFactoryException("Unable to parse definitions from "
-                + resourceString, e);
+            Map<String, String> initParameters) {
+        if (rendererFactory == null) {
+            throw new IllegalStateException("No RendererFactory found");
         }
 
         definitionsFactory.init(initParameters);
 
-        if (LOG.isInfoEnabled()) {
-            LOG.info("Tiles2 container initialization complete.");
+        if (log.isInfoEnabled()) {
+            log.info("Tiles2 container initialization complete.");
         }
     }
 
@@ -418,12 +462,12 @@ public class BasicTilesContainer implements TilesContainer {
      * @since 2.0.6
      */
     @SuppressWarnings("unchecked")
-    protected Stack<AttributeContext> getContextStack(TilesRequestContext tilesContext) {
-        Stack<AttributeContext> contextStack =
-            (Stack<AttributeContext>) tilesContext
+    protected ArrayStack<AttributeContext> getContextStack(TilesRequestContext tilesContext) {
+        ArrayStack<AttributeContext> contextStack =
+            (ArrayStack<AttributeContext>) tilesContext
                 .getRequestScope().get(ATTRIBUTE_CONTEXT_STACK);
         if (contextStack == null) {
-            contextStack = new Stack<AttributeContext>();
+            contextStack = new ArrayStack<AttributeContext>();
             tilesContext.getRequestScope().put(ATTRIBUTE_CONTEXT_STACK,
                     contextStack);
         }
@@ -440,7 +484,7 @@ public class BasicTilesContainer implements TilesContainer {
      */
     protected void pushContext(AttributeContext context,
             TilesRequestContext tilesContext) {
-        Stack<AttributeContext> contextStack = getContextStack(tilesContext);
+        ArrayStack<AttributeContext> contextStack = getContextStack(tilesContext);
         contextStack.push(context);
     }
 
@@ -452,7 +496,7 @@ public class BasicTilesContainer implements TilesContainer {
      * @since 2.0.6
      */
     protected AttributeContext popContext(TilesRequestContext tilesContext) {
-        Stack<AttributeContext> contextStack = getContextStack(tilesContext);
+        ArrayStack<AttributeContext> contextStack = getContextStack(tilesContext);
         return contextStack.pop();
     }
 
@@ -460,12 +504,11 @@ public class BasicTilesContainer implements TilesContainer {
      * Get attribute context from request.
      *
      * @param tilesContext current Tiles application context.
-     * @return BasicAttributeContext or null if context is not found or an
-     *         jspException is present in the request.
+     * @return BasicAttributeContext or null if context is not found.
      * @since 2.0.6
      */
     protected AttributeContext getContext(TilesRequestContext tilesContext) {
-        Stack<AttributeContext> contextStack = getContextStack(tilesContext);
+        ArrayStack<AttributeContext> contextStack = getContextStack(tilesContext);
         if (!contextStack.isEmpty()) {
             return contextStack.peek();
         } else {
@@ -495,10 +538,8 @@ public class BasicTilesContainer implements TilesContainer {
      * @return The created Tiles request context.
      */
     private TilesRequestContext getRequestContext(Object... requestItems) {
-        return getContextFactory().createRequestContext(
-            getApplicationContext(),
-            requestItems
-        );
+        return getRequestContextFactory().createRequestContext(
+                getApplicationContext(), requestItems);
     }
 
     /**
@@ -509,7 +550,12 @@ public class BasicTilesContainer implements TilesContainer {
      */
     private AttributeContext startContext(TilesRequestContext tilesContext) {
         AttributeContext context = new BasicAttributeContext();
-        pushContext(context, tilesContext);
+        ArrayStack<AttributeContext>  stack = getContextStack(tilesContext);
+        if (!stack.isEmpty()) {
+            AttributeContext parent = stack.peek();
+            context.inheritCascadedAttributes(parent);
+        }
+        stack.push(context);
         return context;
     }
 
@@ -529,15 +575,14 @@ public class BasicTilesContainer implements TilesContainer {
      * @param preparerName The name of the preparer.
      * @param ignoreMissing If <code>true</code> if the preparer is not found,
      * it ignores the problem.
-     * @throws TilesException If the preparer is not found (and
+     * @throws NoSuchPreparerException If the preparer is not found (and
      * <code>ignoreMissing</code> is not set) or if the preparer itself threw an
      * exception.
      */
-    private void prepare(TilesRequestContext context, String preparerName, boolean ignoreMissing)
-        throws TilesException {
+    private void prepare(TilesRequestContext context, String preparerName, boolean ignoreMissing) {
 
-        if (LOG.isDebugEnabled()) {
-            LOG.debug("Prepare request received for '" + preparerName);
+        if (log.isDebugEnabled()) {
+            log.debug("Prepare request received for '" + preparerName);
         }
 
         ViewPreparer preparer = preparerFactory.getPreparer(preparerName, context);
@@ -555,117 +600,97 @@ public class BasicTilesContainer implements TilesContainer {
     }
 
     /**
-     * Renders the specified definition.
+     * Renders the definition with specified name.
      *
      * @param request The request context.
      * @param definitionName The name of the definition to render.
-     * @throws TilesException If something goes wrong during rendering.
+     * @throws NoSuchDefinitionException If the definition has not been found.
+     * @throws DefinitionsFactoryException If something goes wrong when
+     * obtaining the definition.
+     * @since 2.1.3
      */
-    private void render(TilesRequestContext request, String definitionName)
-        throws TilesException {
+    protected void render(TilesRequestContext request, String definitionName) {
 
-        if (LOG.isDebugEnabled()) {
-            LOG.debug("Render request recieved for definition '" + definitionName + "'");
+        if (log.isDebugEnabled()) {
+            log.debug("Render request recieved for definition '" + definitionName + "'");
         }
 
         Definition definition = getDefinition(definitionName, request);
 
         if (definition == null) {
-            if (LOG.isWarnEnabled()) {
+            if (log.isWarnEnabled()) {
                 String message = "Unable to find the definition '" + definitionName + "'";
-                LOG.warn(message);
+                log.warn(message);
             }
             throw new NoSuchDefinitionException(definitionName);
         }
+        render(request, definition);
+    }
 
-        if (!isPermitted(request, definition.getRoles())) {
-            if (LOG.isDebugEnabled()) {
-                LOG.debug("Access to definition '" + definitionName
-                        + "' denied.  User not in role '"
-                        + definition.getRoles());
-            }
-            return;
-        }
-
+    /**
+     * Renders the specified definition.
+     * @param request The request context.
+     * @param definition The definition to render.
+     * @since 2.1.3
+     */
+    protected void render(TilesRequestContext request, Definition definition) {
         AttributeContext originalContext = getAttributeContext(request);
         BasicAttributeContext subContext = new BasicAttributeContext(originalContext);
-        subContext.addMissing(definition.getAttributes());
+        subContext.inherit(definition);
+
         pushContext(subContext, request);
 
         try {
-            if (definition.getPreparer() != null) {
-                prepare(request, definition.getPreparer(), true);
-            }
-
-            String dispatchPath = definition.getTemplate();
-
-            if (LOG.isDebugEnabled()) {
-                LOG.debug("Dispatching to definition path '"
-                        + definition.getTemplate() + " '");
-            }
-            request.dispatch(dispatchPath);
-
-            // tiles exception so that it doesn't need to be rethrown.
-        } catch (TilesException e) {
-            throw e;
-        } catch (Exception e) {
-            LOG.error("Error rendering tile", e);
-            // TODO it would be nice to make the preparerInstance throw a more specific
-            throw new TilesException(e.getMessage(), e);
+            render(request, subContext);
         } finally {
             popContext(request);
         }
     }
 
     /**
-     * Calculates the type of an attribute.
+     * Renders an attribute.
      *
-     * @param attr The attribute to check.
-     * @param request The request object.
-     * @return The calculated attribute type.
-     * @throws TilesException If the type is not recognized.
+     * @param attr The attribute to render.
+     * @param requestContext The Tiles request context.
+     * @throws IOException If something goes wrong during rendering.
      */
-    private AttributeType calculateType(Attribute attr,
-            TilesRequestContext request) throws TilesException {
-        AttributeType type;
-        Object valueContent = attr.getValue();
-        if (valueContent instanceof String) {
-            String valueString = (String) valueContent;
-            if (isValidDefinition(request, valueString)) {
-                type = AttributeType.DEFINITION;
-            } else if (valueString.startsWith("/")) {
-                type = AttributeType.TEMPLATE;
-            } else {
-                type = AttributeType.STRING;
-            }
-        } else {
-            type = AttributeType.OBJECT;
+    private void render(Attribute attr, TilesRequestContext requestContext)
+            throws IOException {
+        if (attr == null) {
+            throw new CannotRenderException("Cannot render a null attribute");
         }
 
-        return type;
+        AttributeRenderer renderer = rendererFactory.getRenderer(attr
+                .getRenderer());
+        if (renderer == null) {
+            throw new CannotRenderException(
+                    "Cannot render an attribute with renderer name "
+                            + attr.getRenderer());
+        }
+        renderer.render(attr, requestContext);
     }
 
     /**
-     * Checks if the current user is in one of the comma-separated roles
-     * specified in the <code>role</code> parameter.
+     * Renders the specified attribute context.
      *
      * @param request The request context.
-     * @param roles The list of roles.
-     * @return <code>true</code> if the current user is in one of those roles.
+     * @param attributeContext The context to render.
+     * @throws InvalidTemplateException If the template is not valid.
+     * @throws CannotRenderException If something goes wrong during rendering.
+     * @since 2.1.3
      */
-    private boolean isPermitted(TilesRequestContext request, Set<String> roles) {
-        if (roles == null || roles.isEmpty()) {
-            return true;
+    protected void render(TilesRequestContext request,
+            AttributeContext attributeContext) {
+
+        try {
+            if (attributeContext.getPreparer() != null) {
+                prepare(request, attributeContext.getPreparer(), true);
+            }
+
+            render(attributeContext.getTemplateAttribute(), request);
+        } catch (IOException e) {
+            throw new CannotRenderException(e.getMessage(), e);
         }
-
-        boolean retValue = false;
-
-        for (Iterator<String> roleIt = roles.iterator(); roleIt.hasNext()
-                && !retValue;) {
-            retValue = request.isUserInRole(roleIt.next());
-        }
-
-        return retValue;
     }
 
     /**
