@@ -1,5 +1,5 @@
 /*
- * $Id: CompleteAutoloadTilesContainerFactoryTest.java 821300 2009-10-03 12:16:36Z apetrelli $
+ * $Id: CompleteAutoloadTilesContainerFactoryTest.java 836356 2009-11-15 13:27:43Z apetrelli $
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -72,7 +72,7 @@ import org.junit.Test;
 /**
  * Tests {@link CompleteAutoloadTilesContainerFactory}.
  *
- * @version $Rev: 821300 $ $Date: 2009-10-03 14:16:36 +0200 (sab, 03 ott 2009) $
+ * @version $Rev: 836356 $ $Date: 2009-11-15 14:27:43 +0100 (dom, 15 nov 2009) $
  */
 public class CompleteAutoloadTilesContainerFactoryTest {
 
@@ -319,6 +319,58 @@ public class CompleteAutoloadTilesContainerFactoryTest {
         verify(applicationContext, contextFactory);
     }
 
+    /**
+     * Regression test for TILES-484 issue.
+     *
+     * @throws IOException If something goes wrong.
+     */
+    @Test
+    public void testTILES484first() throws IOException {
+        TilesApplicationContext applicationContext = createMock(TilesApplicationContext.class);
+        TilesRequestContextFactory contextFactory = createMock(TilesRequestContextFactory.class);
+
+        URL url3 = new URL("file:///nonexistent2/tiles.xml");
+
+        Set<URL> urls2 = new HashSet<URL>();
+        urls2.add(url3);
+
+        expect(applicationContext.getResources("/WEB-INF/**/tiles*.xml")).andReturn(null);
+        expect(applicationContext.getResources("classpath*:META-INF/**/tiles*.xml")).andReturn(urls2);
+
+        replay(applicationContext, contextFactory);
+        List<URL> urls = factory.getSourceURLs(applicationContext, contextFactory);
+        assertEquals(1, urls.size());
+        assertTrue(urls.contains(url3));
+        verify(applicationContext, contextFactory);
+    }
+
+    /**
+     * Regression test for TILES-484 issue.
+     *
+     * @throws IOException If something goes wrong.
+     */
+    @Test
+    public void testTILES484second() throws IOException {
+        TilesApplicationContext applicationContext = createMock(TilesApplicationContext.class);
+        TilesRequestContextFactory contextFactory = createMock(TilesRequestContextFactory.class);
+
+        URL url1 = new URL("file:///nonexistent/tiles.xml");
+        URL url2 = new URL("file:///nonexistent/tiles_it.xml");
+
+        Set<URL> urls1 = new HashSet<URL>();
+        urls1.add(url1);
+        urls1.add(url2);
+
+        expect(applicationContext.getResources("/WEB-INF/**/tiles*.xml")).andReturn(urls1);
+        expect(applicationContext.getResources("classpath*:META-INF/**/tiles*.xml")).andReturn(null);
+
+        replay(applicationContext, contextFactory);
+        List<URL> urls = factory.getSourceURLs(applicationContext, contextFactory);
+        assertEquals(1, urls.size());
+        assertTrue(urls.contains(url1));
+        verify(applicationContext, contextFactory);
+    }
+
     /**
      * Test method for
      * {@link CompleteAutoloadTilesContainerFactory
