@@ -1,5 +1,5 @@
 /*
- * $Id: Tiles2ToolTest.java 765774 2009-04-16 21:43:00Z apetrelli $
+ * $Id: Tiles2ToolTest.java 904552 2010-01-29 16:41:26Z apetrelli $
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -455,7 +455,7 @@ public class Tiles2ToolTest {
     }
 
     /**
-     * Test method for {@link org.apache.tiles.velocity.template.Tiles2Tool#getAttribute(java.lang.String)}.
+     * Test method for {@link org.apache.tiles.velocity.template.Tiles2Tool#get(java.lang.String)}.
      */
     @Test
     public void testGetAttribute() {
@@ -469,7 +469,7 @@ public class Tiles2ToolTest {
 
         replay(velocityContext, request, response, servletContext, container, attributeContext);
         initializeTool();
-        assertEquals(attribute, tool.getAttribute("myAttribute"));
+        assertEquals(attribute, tool.get("myAttribute"));
         verify(velocityContext, request, response, servletContext, container, attributeContext);
     }
 
